<?php

namespace app\controllers;

use Yii;
use yii\web\Controller;
use yii\web\Response;
use yii\filters\AccessControl;
use app\models\AccountForm;

class AuthenticationController extends Controller
{

    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::class,
                'rules' => [
                    [
                        'allow' => true,
                        'roles' => ['?'],
                    ]
                ],
            ]
        ];
    }

    public function actionIndex()
    {
        if (Yii::$app->request->isAjax) {
            // AJAX-POST-запрос

            // Инициализация
            $model = new AccountForm(Yii::$app->request->post('AccountForm'));
            $model->scenario = $model::SCENARIO_AUTHENTICATION;

            Yii::$app->response->format = Response::FORMAT_JSON;

            if (!Yii::$app->user->isGuest || $model->authentication()) {
                // Аккаунт аутентифицирован

                // Запись ответа
                $return = [
                    'menu' => (new AccountForm())->deauthenticationGenHtml(),
                    '_csrf' => Yii::$app->request->getCsrfToken()
                ];

                if (($cookies = Yii::$app->request->cookies)->has('redirect')) {
                    // Найдено cookie с переадресацией

                    // Запись ответа
                    $return['redirect'] = '/' . $cookies['redirect'];
                    $return['main'] = $this->renderPartial($return['redirect']  . '/index');

                    // Очистка cookie
                    unset(Yii::$app->response->cookies['redirect']);
                } else {
                    // Не найдено cookie с переадресацией

                    if (Yii::$app->request->pathInfo === 'authentication' || Yii::$app->request->pathInfo === 'registration') {
                        // Если клиент на промежуточном URI

                        // Запись ответа
                        $return['redirect'] = '/';
                        $return['main'] = $this->renderPartial('/index');
                    }
                }

                return $return;
            } else {
                // Аккаунт не аутентифицирован

                Yii::$app->response->statusCode = 400;

                return [
                    'main' => $this->renderPartial('/account', compact('model')),
                    'redirect' => '/authentication',
                    '_csrf' => Yii::$app->request->getCsrfToken()
                ];
            }
        }

        if (!Yii::$app->user->isGuest) {
            Yii::$app->response->redirect('/');
        } else {
            return $this->render('/account');
        }
    }
}
