<?php

namespace app\models;

use carono\exchange1c\interfaces\GroupInterface;
use Zenwalker\CommerceML\Model\Group;

class ProductGroup extends Document implements GroupInterface
{
    public static function collectionName()
    {
        return 'product_group';
    }

    public function attributes()
    {
        return array_merge(
            parent::attributes(),
            ['name', 'onec_id', 'onec_prnt_id']
        );
    }

    public function rules()
    {
        return array_merge(
            parent::rules(),
            [
                // [
                //     'name',
                //     'required',
                //     'message' => 'Заполните поле: {attribute}'
                // ]
            ]
        );
    }

    public function attributeLabels()
    {
        return array_merge(
            parent::attributeLabels(),
            [
                'name' => 'Название (name)',
                'onec_id' => 'Название 1C (onec_id)',
                'onec_prnt_id' => 'Название родителя 1C (onec_prnt_id)',
            ]
        );
    }

    public function writeMember(Document $member, string $group)
    {
        if (isset($member->_key)) {
            return static::writeEdgeBetweenMember($member->collectionName() . '/' . $member->_key, $this->collectionName() . '/' . $group);
        }

        return false;
    }

    /**
     * Создание дерева групп
     * в параметр передаётся массив всех групп (import.xml > Классификатор > Группы)
     * $groups[0]->parent - родительская группа
     * $groups[0]->children - дочерние группы
     *
     * @param Group[] $groups
     */
    public static function createTree1c($groups): Document|null
    {
        foreach ($groups as $group) {
            // Перебор групп

            // Создание
            $parent = static::createByML($group);

            if ($children = $group->getChildren()) {
                // Найден потомок

                // Вход в рекурсию
                $children = static::createTree1c($children);

                // Запись рёбер с родителем
                static::writeEdgeBetweenGroup($parent::collectionName() . '/' . $parent->_key, $children::collectionName() . '/' . $children->_key);

                return $children;
            }

            return $parent;
        }
    }

    /**
     * Создаём группу по модели группы CommerceML
     * проверяем все дерево родителей группы, если родителя нет в базе - создаём
     *
     * @param Group $group
     */
    public static function createByML(Group $group): static|array|null
    {
        if (!$model = static::readByOnecId($group->id)) {
            // Группа не найдена

            // Инициализация
            $model = new static;

            $model->onec_id = $group->id;
        }

        $model->name = $group->name;

        if ($parent = $group->getParent()) {
            // Найден родитель

            // Инициализация (рекурсия)
            $parentModel = static::createByML($parent);

            $model->onec_prnt_id = $parentModel->id;

            unset($parentModel);
        } else {
            $model->onec_prnt_id = null;
        }

        $model->save();

        return $model;
    }

    private static function writeEdgeBetweenMember(string $from, string $to): bool
    {
        // Инициализация
        $edge = new ProductEdgeProductGroup();

        // Настройка
        $edge->_from = $from;
        $edge->_to = $to;

        // Запись
        return $edge->save();
    }

    private static function writeEdgeBetweenGroup(string $from, string $to): bool
    {
        // Инициализация
        $edge = new ProductGroupEdgeProductGroup();

        // Настройка
        $edge->_from = $from;
        $edge->_to = $to;

        // Запись
        return $edge->save();
    }

    public static function readAll()
    {
        return static::find()->all();
    }

    public static function readByName(string $name)
    {
        return static::findOne(['name' => $name]);
    }

    /**
     * Название поля в котором хранится ID из 1C
     */
    public static function getIdFieldName1c(): string
    {
        return 'onec_id';
    }

    public static function readByOnecId(string $onec_id): ?ProductGroup
    {
        return static::findOne(['onec_id' => $onec_id]);
    }
}
