<?php

namespace app\models;

use Yii;
use app\models\Account;
use app\models\Product;
use app\models\SupplyEdgeProduct;
use app\models\traits\Xml2Array;
use carono\exchange1c\interfaces\ProductInterface;

// class Supply extends Product implements OfferInterface
class Supply extends Product implements ProductInterface
{
    use Xml2Array;

    public static function collectionName(): string
    {
        return 'supply';
    }

    public function attributes(): array
    {
        return array_merge(
            parent::attributes(),
            [
                'onec'
            ]
        );
    }

    public function attributeLabels(): array
    {
        return array_merge(
            parent::attributeLabels(),
            [
                'onec' => 'Данные 1C'
            ]
        );
    }

    public function afterSave($data, $vars): void
    {
        // Запись ребра: АККАУНТ -> ПОСТАВКА
        (new AccountEdgeSupply)->write(Yii::$app->user->identity->readId(), $this->readId(), 'import');
    }

    public function setRequisite1c($name, $value): mixed
    {
        return true;
    }

    /**
     * Установка группы, где находится продукт
     */
    public function setGroup1c($group): mixed
    {
        // Чтение группы
        // if ($group = SupplyGroup::readByOnecId($group->id)) {
        //     // Запись ребра: ПОСТАВКА => ГРУППА ПОСТАВОК
        //     return static::writeEdgeBetweenGroup(static::collectionName() . '/' . $this->_key, $group->collectionName() . '/' . $group->_key);
        // }

        return true;
    }

    public static function createProperties1c($properties): mixed
    {
        return true;
    }

    public function setProperty1c($property): mixed
    {
        return true;
    }

    public function addImage1c($path, $caption): mixed
    {
        return true;
    }

    /**
     * Запись ребра (предложения от поставки к продукту)
     */
    public function getOffer1c($offer): SupplyEdgeProduct
    {
        if (empty($this->catn)) {
            // Не передан каталожный номер

            // Разработчику библеотеки надо дать по жопе
            return new SupplyEdgeProduct;
        } else if (!$catn = Product::readByCatn($this->catn)) {
            // Продукт не найден

            if (!$this->initProduct()) {
                // Не удалось инициализировать продукт

                // Разработчику библеотеки надо дать по жопе
                return new SupplyEdgeProduct;
            }
        }

        $catn = Product::readByCatn($this->catn);

        // Запись ребра: ПОСТАВКА -> ПРОДУКТ
        return (new SupplyEdgeProduct)->write(
            $this->readId(),
            $catn->readId(),
            'sell',
            [
                'onec' => self::xml2array($offer->xml)
            ]
        );
    }

    /**
     * Создать продукт
     */
    public static function createModel1c($product): self
    {
        // Инициализация
        $model = self::readByOnecId($id = (string) $product->Ид) ?? new self;

        // Настройки
        $model->ocid = $id ?? null;
        $model->catn = (string) $product->Артикул;
        $model->oemn = null;
        $model->onec = self::xml2array($product->xml);

        // Запись
        $model->save();

        return $model;
    }

    protected function initProduct(): bool
    {
        // Надо не забыть сделать выборку полей и ручное подключение

        if (empty($this->catn)) {
            // Не передан каталожный номер

            return false;
        } else if (Product::readByCatn($this->catn)) {
            // Продукт уже был инициализирован

            return true;
        }

        // Инициализация
        $product = new Product();

        // Настройки
        $product->catn = $this->catn;

        // Запись
        return $product->save();
    }


    public function setPrice1c($price): void
    {
    }

    public function setRaw1cData($cml, $object): bool
    {
        return false;
    }

    public static function readByOnecId(string $ocid): ?Supply
    {
        return self::findOne([self::getIdFieldName1c() => $ocid]);
    }

    public function getGroup1c(): SupplyGroup
    {
        return $this->group;
    }

    /**
     * Название поля в котором хранится ID из 1C
     */
    public static function getIdFieldName1c(): string
    {
        return 'ocid';
    }
}
